import re
from string import ascii_uppercase, ascii_lowercase
import hashlib
from django.db import models
import pgpy

from . import zbase32

lower_trans = str.maketrans(ascii_uppercase, ascii_lowercase)
def lower(value):
    return value.translate(lower_trans)

class PublicKey(models.Model):
    email = models.CharField(max_length=128)
    armor = models.TextField()
    keyid = models.CharField(max_length=40, null=True, blank=True)
    wkdid = models.CharField(max_length=32, null=True, blank=True)

    def __init__(self, *args, **kwds):
        super().__init__(*args, **kwds)
        self.decoded = None

    def set_keyid(self):
        if not self._decode():
            return
        fingerprint = self.decoded.fingerprint
        self.keyid = re.sub("\s+", "", fingerprint).lower()

    def wkddomain(self):
        local, domain = self.email.rsplit("@", 1)
        return domain.lower()

    def set_wkdid(self):
        self.wkdid = self.encode_wkd_local(self.email)

    @staticmethod
    def encode_wkd_local(email):
        local, domain = email.rsplit("@", 1)
        local = lower(local)
        digest = hashlib.sha1(local.encode()).digest()
        return zbase32.encode(digest).decode()

    def save(self, *args, **kwds):
        if not self.keyid:
            self.set_keyid()

        if not self.wkdid:
            self.set_wkdid()
 
        super().save(*args, **kwds)

    def _decode(self):
        if self.decoded:
            return True

        try:
            self.decoded, _ = pgpy.PGPKey.from_blob(self.armor)
        except ValueError:
            return False

        return True

    def details(self):
        if not self._decode():
            return None

        def get_exp(created, expires_at):
            if not expires_at:
                return None
            return created + expires_at

        data = {
            "userids": [{
                "email": userid.email,
                "name": userid.name,
                "comment": userid.comment,
                "signatures": [{
                    "created": sig.created,
                    "expires_at": get_exp(sig.created, sig.expires_at),
                    "signer": sig.signer,
                } for sig in userid._signatures],
            } for userid in self.decoded.userids],
            "fingerprint": self.decoded.fingerprint.split(" "),
            "key_size": self.decoded.key_size,
            "id": re.sub("\s+", "", self.decoded.fingerprint[-8:]),
            "created": self.decoded.created,
            "expires_at": self.decoded.expires_at,
            "signatures": [{
                "created": sig.created,
                "expires_at": get_exp(sig.created, sig.expires_at),
                "signer": sig.signer,
            } for sig in self.decoded._signatures],
            "subkeys": [{
                "fingerprint": subkey.fingerprint.split(" "), 
                "created": subkey.created,
                "expires_at": subkey.expires_at,
                "signatures": [{
                    "created": sig.created,
                    "expires_at": get_exp(subkey.created, sig.key_expiration),
                    "signer": sig.signer,
                }
                for sig in subkey._signatures
                if sig.type == pgpy.constants.SignatureType.Subkey_Binding],
            } for id, subkey in self.decoded.subkeys.items()],
        }
        return data

    def __str__(self):
        keyid = f" (0x{self.keyid[-8:]})" if self.keyid else ""
        return f"{self.email}{keyid}"
